#!/usr/bin/env python
# -*- coding: utf-8 -*-
from builtins import str
import json
import os

from cloud_resources import storage_catalyst
from db_samvaad import db_credentials
from db_samvaad.db_handler import DatabaseHandle
import db_samvaad as db_con

from importlib import reload
from lambda_wrapper import constants as cnts
from lambda_wrapper import credentials as creds

def init_lambda(event, context=None):
    '''
    This is the function which is called from the lambda_algo_boiler plate.
    It initializes the environment to let the lambda function work.

    PARAMETERS
    ------------
    event : dict
        Object that contains all the parameters passed while invoking the lambda function
    context : LambdaContext
        object that contains the runtime information about the lambda function.
    For more information refer :
        http://docs.aws.amazon.com/lambda/latest/dg/python-programming-model-handler-types.html

    '''

    body = json.loads(event["body"])
    iter_id = str(body["iter_id"])
    env_type = str(body["env_type"])

    set_envs(iter_id, env_type)
    download_pem_file()

    db_con.db.update_state(iter_id, 'on_lambda')

def set_envs(iter_id, env_type):
    '''
    This function sets all the environment variables which are needed for the execution
    of the lambda function.
    Environment variables are fetched from envconfig file which is present  in s3

    PARAMETERS
    ---------------
    iter_id : int
        It is the iteration id

    env_type : str
        It is the type of environment for which the lambda function is executing
    '''

    os.environ[cnts.ITER_ID] = iter_id
    env_data = fetch_env_config()
    env_configs = env_data.get(env_type)
    if env_configs:
        for env_name, env_val in list(env_configs.items()):
            os.environ[env_name] = env_val

        reload(db_credentials)
        reload(creds)
        override(iter_id)
        reload(creds)
        reload(db_credentials)

        parameter_data = get_param_json(iter_id)
        os.environ[cnts.PARAMETER_KEY] = str(parameter_data)
    else:
        raise Exception(cnts.INVALID_ENV)

def override(iter_id):
    '''
    This function is used to maps the overriding environment variable to
    overriden env. It is used to override the environment variable values.

    PARAMETERS
    -----------
    iter_id : int
        iteration id

    '''
    mapper = {}
    mapper[cnts.MOUNT_FILE_PATH] = cnts.OUTPUT_FILE_PATH
    mapper[cnts.SIGNING_PEM_FILE_PATH] = cnts.SIGNING_PEM_FILE_TMP_PATH

    if iter_type(iter_id) == cnts.ITR_TYPE_WF:
        mapper[cnts.USER_INPUT_S3_BUCKET] = cnts.USER_WF_INPUT_S3_BUCKET
        mapper[cnts.ITER_OUTPUT_S3_BUCKET] = cnts.ITER_WF_OUTPUT_S3_BUCKET
        mapper[cnts.USER_INPUT_CF_URL] = cnts.USER_WF_INPUT_CF_URL
        mapper[cnts.ITER_OUTPUT_CF_URL] = cnts.ITER_WF_OUTPUT_CF_URL
        
    override_params(mapper)

def override_params(mapper):
    '''
    To override the env variables using mapper Dict

    PARAMETERS
    ----------
    mapper : Dict

    '''
    for key, value in list(mapper.items()):
        os.environ[key] = os.getenv(value)

def iter_type(iter_id):
    '''
    To check whether the current iteration is a part of workflow or not

    PARAMETERS
    -----------
    iter_id : int
        iteration id

    RETURNS
    -------
    boolean:
        boolean representing whether this iteration belongs to workflow
        or not
    '''
    db_con.db = DatabaseHandle()
    query = 'select type from iteration where id={}'
    arguments = (iter_id,)
    iter_type = db_con.db.execute_query(query, arguments)

    return iter_type[0][0]

def get_param_json(iter_id):
    '''
        This function is used to fetch the data of paramter_json associtated with the
        given iter_id

    PARAMETERS
    -------------
    iter_id : int
        Iteration id

    RETURNS
    --------------
    parameter_data : Dict
        data of the paramter_json file for the iteration

    '''

    db_con.db = DatabaseHandle()
    iteration_type = iter_type(iter_id)
    if iteration_type == cnts.ITR_TYPE_WF:
        parent_id = db_con.db.get_value_from_iter_id(iter_id, cnts.WORKFLOW_REQUEST)
    elif iteration_type == cnts.ITR_TYPE_CMP:
        parent_id = db_con.db.get_value_from_iter_id(iter_id, cnts.RUN)
    else:
        raise AttributeError(cnts.INVALID_ITER_TYPE)

    s3_key = str(parent_id) + os.sep + str(iter_id) + os.sep + str(cnts.PARAMETER_JSON)
    parameter_data = storage_catalyst.get_object(s3_key,creds.USER_BUCKET)
    return parameter_data

def fetch_env_config():
    '''
    This function get the envconfig file data. This file contains the environment
    variables and their values which are needed to run the algorithm.

    '''
    env_data = storage_catalyst.get_object(cnts.ENV_FILE_KEY,creds.ENVRIOMENT_BUCKET)
    env_data = json.loads(env_data)
    return env_data

def update_iter_state():
    '''
    This function is to update the state of iteration. It checks if the
    state of the iteration is not set to 'completed' then it sets it to
    'error'

    '''
    iter_id = os.getenv(cnts.ITER_ID)
    db_con.db = DatabaseHandle()
    state = db_con.db.get_value_from_iter_id(iter_id, 'state')

    response = 1
    final_state = 'completed'

    if state != 'completed':
        final_state = 'error'
        response = db_con.db.update_state(iter_id, final_state)

    return final_state if response else None


def download_pem_file():
    '''
    This function is used to download the .pem file for cloud front. This
    file is needed to sign the cloudfront url.

    '''
    output_path = os.path.dirname(creds.PEM_FILE_PATH)
    
    os.path.isdir(output_path) or os.mkdir(output_path)

    pem_file_name = os.path.basename(creds.PEM_FILE_PATH)
    bucket = creds.ENVRIOMENT_BUCKET
    pem_file_content = storage_catalyst.get_object(pem_file_name, bucket)

    with open(creds.PEM_FILE_PATH,'w') as fout:
        fout.write(str(pem_file_content))
   